<?php

namespace App\Domain\Imports\Writers\Sheets;

use Maatwebsite\Excel\Concerns\RegistersEventListeners;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;

class ProductsSheet implements WithEvents, WithHeadings, WithTitle
{
    use RegistersEventListeners;

    private array $headings;
    private ?int $categoryId;
    public const MAX_ROWS = 1048576;

    public function __construct(?int $categoryId)
    {
        $this->categoryId = $categoryId;
        $this->headings = [
            'brand_id' => __('import.brand_id'),
            'category_ids' => __('import.category_ids'),
            'barcode' => __('import.barcode'),
            'name' => __('import.name'),
            'code' => __('import.code'),
            'description' => __('import.description'),
            'weight' => __('import.weight'),
            'width' => __('import.width'),
            'height' => __('import.height'),
            'length' => __('import.length'),
            'weight_gross' => __('import.weight_gross'),
            'allow_publish' => __('import.allow_publish'),
            'vendor_code' => __('import.vendor_code'),
            'is_adult' => __('import.is_adult'),
            'type' => __('import.type'),
            'image' => __('import.image'),
        ];
    }

    public function headings(): array
    {
        return array_keys($this->headings);
    }

    public static function afterSheet(AfterSheet $event): void
    {
        /** @var static $thisSheet */
        $thisSheet = $event->getConcernable();
        $sheet = $event->getSheet();

        foreach ($sheet->getColumnIterator() as $column) {
            $columnIndex = $column->getColumnIndex();
            $cell = $sheet->getCell("{$columnIndex}1");

            $validation = $cell->getDataValidation();
            $validation->setShowInputMessage(true);
            $validation->setPrompt($thisSheet->headings[$cell->getValue()]);

            if ($cell == 'category_ids') {
                $validation = $sheet->getCell("{$columnIndex}2")->getDataValidation();

                $validation->setType(DataValidation::TYPE_LIST);
                $validation->setAllowBlank(false);
                $validation->setShowDropDown(true);
                $validation->setShowErrorMessage(true);
                $validation->setFormula1((string)$thisSheet->categoryId);

                $validation->setSqref("{$columnIndex}2:{$columnIndex}" . self::MAX_ROWS);
            }
        }
    }

    public function title(): string
    {
        return __('import.products_attributes');
    }
}
